import discord
from discord import Role, TextChannel
from discord.ext import commands
from discord_slash import cog_ext
from discord_slash.utils.manage_commands import create_option

import jarvis
from jarvis.config import get_config
from jarvis.utils import db
from jarvis.utils.permissions import admin_or_permissions


class SettingsCog(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        config = get_config()
        self.db = db.DBManager(config.mongo).mongo
        self.cache = {}

    def update_settings(self, setting, value, guild):
        settings = {"setting": setting, "value": value, "guild": guild}
        updated = self.db.jarvis.settings.update_one(
            {"settings": setting, "guild": guild},
            {"$set": settings},
            upsert=True,
        )

        return updated is not None

    @cog_ext.cog_subcommand(
        base="settings",
        name="mute",
        description="Set mute role",
        options=[
            create_option(
                name="role",
                description="Mute role",
                option_type=8,
                required=True,
            )
        ],
    )
    @admin_or_permissions(mute_members=True)
    async def _mute(self, ctx, role: Role):
        await ctx.defer()
        self.update_settings("mute", role.id, ctx.guild.id)
        await ctx.send(f"Settings applied. New mute role is `{role.name}`")

    @cog_ext.cog_subcommand(
        base="settings",
        name="modlog",
        description="Set modlog channel",
        options=[
            create_option(
                name="channel",
                description="Modlog channel",
                option_type=7,
                required=True,
            )
        ],
    )
    @commands.has_permissions(administrator=True)
    async def _modlog(self, ctx, channel: TextChannel):
        await ctx.defer()
        self.update_settings("modlog", channel.id, ctx.guild.id)
        await ctx.send(
            f"Settings applied. New modlog channel is {channel.mention}"
        )

    @cog_ext.cog_subcommand(
        base="settings",
        name="userlog",
        description="Set userlog channel",
        options=[
            create_option(
                name="channel",
                description="Userlog channel",
                option_type=7,
                required=True,
            )
        ],
    )
    @commands.has_permissions(administrator=True)
    async def _userlog(self, ctx, channel: TextChannel):
        await ctx.defer()
        self.update_settings("userlog", channel.id, ctx.guild.id)
        await ctx.send(
            f"Settings applied. New userlog channel is {channel.mention}"
        )


def setup(bot):
    bot.add_cog(SettingsCog(bot))
